import time
import tensorflow as tf
import warnings

if tf.__version__[0] == 1:
    SessionRunArgs = tf.train.SessionRunArgs
    SessionRunHook = tf.train.SessionRunHook
else:
    SessionRunArgs = tf.compat.v1.train.SessionRunArgs
    SessionRunHook = tf.estimator.SessionRunHook

NONE_INDEX = -333


class LoggerHook(SessionRunHook):
    """Logs loss and runtime."""

    def set_run(self, run, loss, phase="train"):
        warnings.warn("Native TF plugins are not supported yet.")
        return
        self.run = run
        self.phase = phase
        self.loss = loss
        self._epoch = -1
        run.init_ml()
        run.start_ml()

    def begin(self):
        self._epoch = -1

    def before_run(self, run_context):
        self._epoch += 1
        # Asks for loss value.
        return SessionRunArgs(self.loss)

    def after_run(self, run_context, run_values):
        loss_value = run_values.results
        self.run.log_ml(epoch=self._epoch, loss=loss_value.astype(
            float), phase=self.phase)
        #print("DEBUG: EPOCH {}, LOSS VALUE: {}".format(self._epoch, loss_value))

    def end(self, session):
        self.run.conclude()



from tensorflow import keras
import os


LOGS_VAL = "val_"
    

class MWCustomCallback(keras.callbacks.Callback):
    def set_run(self, run):
        self.run = run
        self.test_epoch = 0
        run.init_ml()
        run.start_ml()

    def conclude_run(self):
        self.run.conclude()

    def on_test_batch_end(self, batch, logs=None):
        pass

    def on_epoch_end(self, epoch, logs=None):
        loss = logs.get('loss')
        acc = logs.get('acc')
        acc = acc if acc else logs.get('accuracy')
        
        self._log_val_if_exists(epoch, logs=logs)
        self.run.log_ml(epoch=epoch, loss=loss,
                        acc=acc, phase="train", custom_logs=logs)

    def _log_val_if_exists(self, epoch, logs=None):
        loss = logs.get(LOGS_VAL+'loss')
        acc = logs.get(LOGS_VAL+'acc')
        acc = acc if acc else logs.get(LOGS_VAL+'accuracy')
        if loss and acc:
            self.run.log_ml(epoch=epoch, loss=loss,
                            acc=acc, phase="val", custom_logs=logs)

    # def on_train_begin(self, logs):

    def on_test_begin(self, logs=None):
        # todo
        self.run.start_ml()

    def on_test_end(self, logs=None):
        self.test_epoch += 1
        loss = logs.get('loss')
        acc = logs.get('acc')
        acc = acc if acc else logs.get('accuracy')
        self.run.log_ml(epoch=self.test_epoch, loss=loss,
                        acc=acc, phase="test")
        # self.run.conclude()

    def on_train_end(self, logs):
        pass
        # upload/save to local
        # self.run.conclude()



class LoggerHook:
    def set_run(self, run, phase="train"):
        self.run = run
        self.phase = phase
        run.init_ml()
        run.start_ml()
        self._step = -1

    def conclude_run(self):
        self.run.conclude()
        self._step = -1

    def torch_loss_hook(self, model, input, loss):
        loss_val = -1
        if isinstance(loss.item(), list):
            loss_val = loss.item()[0]
        else:
            loss_val = loss.item()
        self._step += 1
        self.run.log_ml(step=self._step, loss=loss_val, phase=self.phase)



import mwutils
from mwutils.logs import Logger, mili_time
from mwutils.sys_stat import SystemStats
import os
import requests
import warnings
import jwt
import time
import traceback
import signal
import json
import numpy as np
from os import path, getpid
from random import randrange

_STEP = 'step'
_EPOCH = 'epoch'
_BATCH = 'batch'
_LOSS = 'loss'
_ACC = 'acc'
_TIMESTAMP = 'timestamp'

_MAX_ACC = "max_accuracy"
_MIN_LOSS = "min_loss"
_BEST = "best"

MODEL_TYPE_TF = "tf"
MODEL_TYPE_KERAS = "keras"
MODEL_TYPE_TORCH = "torch"
MODEL_TYPE_CUSTOM = "custom"

run_names = {}


class MLLoger(Logger):
    def log(self, step=None, epoch=None, batch=None, loss=None, acc=None, custom_logs=None):
        val = dict()
        val[_TIMESTAMP] = int(time.time())
        if step is not None:
            val[_STEP] = step + 1
        if epoch is not None:
            val[_EPOCH] = epoch + 1
        if batch is not None:
            val[_BATCH] = batch + 1
        if loss is not None:
            val[_LOSS] = loss
        if acc is not None:
            val[_ACC] = acc

        if acc:
            if _MAX_ACC not in self.memoize and acc:
                self.memoize[_MAX_ACC] = val
            elif self.memoize[_MAX_ACC][_ACC] < val[_ACC]:
                self.memoize[_MAX_ACC] = val

        if loss:
            best = False
            if _MIN_LOSS not in self.memoize and loss:
                self.memoize[_MIN_LOSS] = val
                best = True
            elif self.memoize[_MIN_LOSS][_LOSS] > val[_LOSS]:
                self.memoize[_MIN_LOSS] = val
                best = True
            if best:
                if step is not None:
                    self.memoize["{}_{}".format(_BEST, _STEP)] = step+1
                elif epoch is not None:
                    self.memoize["{}_{}".format(_BEST, _EPOCH)] = epoch+1
                elif batch is not None:
                    self.memoize["{}_{}".format(_BEST, _BATCH)] = batch+1
        if custom_logs:
            if isinstance(custom_logs, dict):
                for k, v in custom_logs.items():
                    if k not in ['loss', 'acc', 'accuracy', 'val_loss', 'val_acc', 'val_accuracy']:
                        if 'custom_keys' not in self.metadata['annotations']:
                            self.metadata['annotations']['custom_keys'] = []
                        if k not in self.metadata['annotations']['custom_keys']:
                            self.metadata['annotations']['custom_keys'].append(
                                k)
                        val[k] = v
        for k, _ in val.items():
            if k not in self.metadata['annotations']['keys']:
                self.metadata['annotations']['keys'].append(k)
        super().log(val)


def example_memoize_func(memoize_buf, val):
    if "cost" in val:
        if "min_cost" in memoize_buf and memoize_buf["min_cost"] > val["cost"]:
            memoize_buf["min_cost"] = val["cost"]


class CustomLogger(Logger):
    pass


class Run():
    def __init__(self, name="", user_id="", lab_id="", org_id="", flush_interval_seconds=5,
                 sys_stat_sample_size=1, sys_stat_sample_interval=2, local_path='', write_logs_to_local=False,
                 remote_path='', buffer_all_logs=False):
        if name == '':
            name == '数据科学实验@' + str(randrange(999))
        if name in run_names:
            s = "name {} is already used in current session.".format(name)
            raise Exception(s)
        p = os.path.expanduser('~')
        _path = p + '/.ide/config.json'
        _data = None
        config_user_id = None
        config_lab_id = None
        config_org_id = None
        if os.path.exists(_path):
            f = open(_path)
            _data = json.load(f)
            f.close()
        if _data:
            _remote_path = _data['website']['siteUrl']
            config_user_id = _data['website']['user']['_id']
            config_lab_id = _data['website']['lab']['_id']
            config_org_id = _data['website']['org']['_id']
        run_names[name] = self
        self._loggers = {}
        self.custom_loggers = {}
        env_user_id = os.getenv("ENV_USER_ID")
        env_lab_id = os.getenv("ENV_LAB_ID")
        env_org_id = os.getenv("ENV_ORG_ID")
        if config_user_id:
            self.user_id = config_user_id
        elif env_user_id:
            self.user_id = env_user_id
        else:
            self.user_id = user_id
        if config_lab_id:
            self.lab_id = config_lab_id
        elif env_lab_id:
            self.lab_id = env_lab_id
        else:
            self.lab_id = lab_id
        if config_org_id:
            self.org_id = config_org_id
        elif env_org_id:
            self.org_id = env_org_id
        else:
            self.org_id = org_id

        if remote_path:
            self.remote_path = remote_path
        elif _remote_path:
            self.remote_path = _remote_path + '/api/runs'
        else:
            self.remote_path = 'https://www.heywhale.com/api/runs'

        print('remotepath is', self.remote_path)
        timestr = str(mili_time())
        if not (self.user_id and self.lab_id and self.org_id):
            s = "At least one of required fields is empty:\nuser_id: {}\norg_id: {}\nlab_id: {}\n".format(
                user_id, org_id, lab_id)
            raise Exception(s)
        self.run_id = name + '_' + timestr
        self.flush_interval_seconds = max(5, flush_interval_seconds)
        self._sys_stat_sample_size = sys_stat_sample_size
        self._sys_stat_sample_interval_seconds = sys_stat_sample_interval
        self.local_path = local_path
        self.write_logs_to_local = write_logs_to_local
        self.logs_remote_path = remote_path + '/logs' if remote_path else ''
        self.conclude_remote_path = remote_path + '/conclude'
        self.abort_remote_path = remote_path + "/abort"
        self.buffer_all_logs = buffer_all_logs
        self.model_path = ""
        self.metadata = {"name": name, "user_id": user_id,
                         "lab_id": lab_id, "run_id": self.run_id, "org_id": org_id, "annotations": {"custom_keys": [], "keys": []}}
        self.pid = None
        self.started = False

    def init_ml(self):
        if self.pid:
            return
        self.pid = getpid()
        train_path = path.join(
            self.local_path, "train.json") if self.write_logs_to_local else ''
        test_path = path.join(
            self.local_path, "test.json") if self.write_logs_to_local else ''
        val_path = path.join(
            self.local_path, "val.json") if self.write_logs_to_local else ''
        sys_path = path.join(
            self.local_path, "sys.json") if self.write_logs_to_local else ''
        self._loggers['train'] = MLLoger("train", sample_time_interval_seconds=self.flush_interval_seconds,
                                         metadata=self.metadata, local_path=train_path, post_addr=self.logs_remote_path,
                                         buffer_all=self.buffer_all_logs)
        self._loggers['test'] = MLLoger("test", sample_time_interval_seconds=self.flush_interval_seconds,
                                        metadata=self.metadata, local_path=test_path, post_addr=self.logs_remote_path,
                                        buffer_all=self.buffer_all_logs)
        self._loggers['val'] = MLLoger("val", sample_time_interval_seconds=self.flush_interval_seconds,
                                       metadata=self.metadata, local_path=val_path, post_addr=self.logs_remote_path,
                                       buffer_all=self.buffer_all_logs)
        self._loggers['system'] = CustomLogger("system", sample_time_interval_seconds=self.flush_interval_seconds,
                                               metadata=self.metadata, local_path=sys_path, post_addr=self.logs_remote_path,
                                               buffer_all=self.buffer_all_logs)
        self._loggers['meta'] = CustomLogger("meta", sample_time_interval_seconds=self.flush_interval_seconds,
                                             metadata=self.metadata, local_path=sys_path, post_addr=self.logs_remote_path,
                                             buffer_all=self.buffer_all_logs)

    def start_ml(self):
        if self.started:
            return
        self.started = True
        self.__register_signal_handlers()
        for _, logger in self._loggers.items():
            logger.start()
        for _, clogger in self.custom_loggers.items():
            clogger.start()
        self.sys_stat = SystemStats(self)
        self.sys_stat.start()

    def log_meta(self, data):
        self._loggers['meta'].log(data)

    def log_ml(self, step=None, epoch=None, batch=None, loss=None, acc=None, phase="train", custom_logs=None):
        # phase is the same thing with namea
        try:
            acc = float(acc)
            loss = float(loss)
        except:
            raise TypeError('acc or loss cannot be transferred to float!')
     
        self._loggers[phase].log(step=step, epoch=epoch,
                                 batch=batch, loss=loss, acc=acc, custom_logs=custom_logs)

    def new_custom_logger(self, name, local_path=''):
        self.custom_loggers[name] = CustomLogger(name, sample_time_interval_seconds=self.flush_interval_seconds,
                                                 metadata=self.metadata, local_path=local_path, post_addr=self.logs_remote_path,
                                                 buffer_all=self.buffer_all_logs)

    def add_memoize_funcs_to_logger(self, name, funcs):
        self._loggers[name].add_memoize_funcs(funcs)

    def set_tf_model(self, model):
        self.model = model
        self.model_type = MODEL_TYPE_TF

    def _save_tf_model(self, model_path):
        # SavedModel
        # tf2
        import tensorflow as tf
        tf.saved_model.save(self.model, model_path)
        pass

    def set_keras_model(self, model):
        self.model = model
        self.model_type = MODEL_TYPE_KERAS

    def _save_keras_model(self, model_path):
        # SavedModel
        # tf2
        import tensorflow as tf
        tf.keras.models.save_model(self.model, model_path)
        self.model_path = model_path

    def set_torch_model(self, model):
        self.model = model
        self.model_type = MODEL_TYPE_TORCH

    def _save_torch_model(self, model_path):
        # torch version >= 1.6
        import torch
        torch.save(self.model, model_path)
        self.model_path = model_path
        pass

    def set_custom_model(self, path):
        self.model_type = MODEL_TYPE_CUSTOM
        self.model_path = path

    def _save_model(self, model_path):
        if hasattr(self, "model_type"):
            if self.model_type == MODEL_TYPE_TORCH:
                self._save_torch_model(model_path)
            elif self.model_type == MODEL_TYPE_KERAS:
                self._save_keras_model(model_path)
            elif self.model_type == MODEL_TYPE_TF:
                self._save_tf_model(model_path)

    def __upload_model(self):
        pass

    def __register_signal_handlers(self):
        signal.signal(signal.SIGINT, self.__sigint_handler)
        signal.signal(signal.SIGTERM, self.__sigterm_handler)

    def __sigint_handler(self, signum, frame):
        self.__abort_run("SIGINT", "[SIGINT]Terminated by system")
        traceback.print_stack(f=frame)
        raise RuntimeError("terminated by system")

    def __sigterm_handler(self, signum, frame):
        self.__abort_run("SIGTERM", "[SIGTERM]Terminated by user")
        traceback.print_stack(f=frame)
        raise KeyboardInterrupt("termniated by user")

    def __abort_run(self, sig, reason):
        if self.remote_path:
            tp = int(time.time())
            json_struct = {"metadata": self.metadata,
                           "timestamp": tp, "signal": sig, "reason": reason}
            for _ in range(3):
                r = requests.post(self.abort_remote_path, json=json_struct, headers={"Authorization": jwt.encode(
                    {"whatever": "1"}, "857851b2-c28c-4d94-83c8-f607b50ccd03")})
                if r.status_code >= 400:
                    # something wrong
                    jb = ''
                    try:
                        jb = r.json()
                    except:
                        pass
                    print("resp:", r)
                    msg = "code: {}, resp.json: {}, resp.text: {}".format(
                        r.status_code, jb, r.text)
                    print(msg)
                    warnings.warn(msg)
                else:
                    print("abort remote call succeed. resp:", r)
                    break
        self.started = False
        self.run_id = "aborted"

    def conclude(self, show_memoize=True, upload_model=False, model_path="./saved_model"):
        if not self.started:
            pass
        for _, logger in self._loggers.items():
            logger.cancel()
            if show_memoize and logger.memoize:
                print(logger.name, logger.memoize)
        for _, clogger in self.custom_loggers.items():
            clogger.cancel()
            if show_memoize and clogger.memoize:
                print(clogger.name, clogger.memoize)
        self._save_model(model_path)

        if self.remote_path:
            tp = int(time.time())
            json_struct = {"metadata": self.metadata, "best": [
                {"phase": name, "val": logger.memoize, _TIMESTAMP: tp} for name, logger in self._loggers.items()]}
            for _ in range(3):
                r = requests.post(self.conclude_remote_path, json=json_struct, headers={"Authorization": jwt.encode(
                    {"whatever": "1"}, "857851b2-c28c-4d94-83c8-f607b50ccd03")})
                if r.status_code >= 400:
                    # something wrong
                    jb = ''
                    try:
                        jb = r.json()
                    except:
                        pass
                    print("resp:", r)
                    msg = "code: {}, resp.json: {}, resp.text: {}".format(
                        r.status_code, jb, r.text)
                    print(msg)
                    warnings.warn(msg)
                else:
                    print("conclude remote call succeed. resp:", r)
                    break
        if upload_model:
            self.__upload_model()
        self.started = False
        self.run_id = "concluded"


if __name__ == "__main__":
    import time

    def sys_memoize_func_maxcpu(memoize_buf, val):
        if "cpu" in val:
            if "max_cpu" in memoize_buf and memoize_buf["max_cpu"] < val["cpu"]:
                memoize_buf["max_cpu"] = val["cpu"]
            if "max_cpu" not in memoize_buf:
                memoize_buf["max_cpu"] = val["cpu"]

    def sys_memoize_func_mincpu(memoize_buf, val):
        if "cpu" in val:
            if "min_cpu" in memoize_buf and memoize_buf["min_cpu"] > val["cpu"]:
                memoize_buf["min_cpu"] = val["cpu"]
            if "min_cpu" not in memoize_buf:
                memoize_buf["min_cpu"] = val["cpu"]

    r = Run("test88", "testuser123", "proj123", "job123", flush_interval_seconds=5,
            local_path="/Users/mk/heyw/github/mwutils/mwutils", sys_stat_sample_interval=5, sys_stat_sample_size=21, buffer_all_logs=True)
    r.init_ml()
    r.add_memoize_funcs_to_logger(
        "system", [sys_memoize_func_maxcpu, sys_memoize_func_mincpu])

    r.start_ml()

    for i in range(150):
        r.log_ml(step=i, acc=i*(1/150), loss=149-i)
        time.sleep(0.2)
    for i in range(20):
        r.log_ml(epoch=i, acc=i*(1/40)+0.5, loss=1, phase='test')
        time.sleep(0.2)
    r.conclude()



import threading
import time
import os
import requests
import warnings
import json
import atexit
import jwt


def mili_time():
    return int(time.time() * 1000)


def nano_time():
    return int(time.time() * 1000 * 1000)


def append_logs_buf_to_local(logs_buf, filename):
    with open(filename, 'a+') as f:
        for entry in logs_buf:
            f.write(json.dumps(entry))


def append_logs_buf_to_remote(logs_buf, name, metadata, post_addr):
    json_struct = {"logs": logs_buf, "phase": name, "metadata": metadata}
    # try 3 times
    for _ in range(3):
        r = requests.post(post_addr, json=json_struct, headers={"Authorization": jwt.encode(
            {"whatever": "1"}, "857851b2-c28c-4d94-83c8-f607b50ccd03")})
        if r.status_code >= 400:
            # something wrong
            jb = ''
            try:
                jb = r.json()
            except:
                pass
            print("resp:", r)
            warnings.warn("code: {}, resp.json: {}, resp.text: {}".format(
                r.status_code, jb, r.text))
        else:
            return True
    return False


class RepeatedTimer(object):
    def __init__(self, interval, function, *args, **kwargs):
        self._timer = None
        self.interval = interval
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.is_running = False
        self.start()

    def _run(self):
        self.is_running = False
        self.start()
        self.function(*self.args, **self.kwargs)

    def start(self):
        if not self.is_running:
            self._timer = threading.Timer(self.interval, self._run)
            self._timer.start()
            self.is_running = True

    def stop(self):
        self._timer.cancel()
        self.is_running = False


class Logger():
    # Logger for metrics
    def __init__(self, name, sample_time_interval_seconds=10, metadata={}, local_path="", post_addr="", buffer_all=False):
        self._logs_buf = []
        self._sample_time_interval_seconds = max(sample_time_interval_seconds, 2)
        self.name = name
        self.metadata = metadata
        self.mutex = threading.Lock()
        self._thread = RepeatedTimer(
            self._sample_time_interval_seconds, self.flush)
        self.post_addr = post_addr
        self.remote = True if post_addr else False
        self.local_path = local_path
        self._user_buf = list()
        self._buffer_all = buffer_all
        self.memoize = dict()
        self.memoize_funcs = []

        # flush before exit
        atexit.register(self.flush)

    def append(self, val):
        with self.mutex:
            q = {"val": val, "timestamp": mili_time()}
            self._logs_buf.append(q)
            if self._buffer_all:
                self._user_buf.append(q)

    def start(self):
        # do not delete existing file,
        if self.local_path:
            with open(self.local_path, 'w+'):
                pass
        self._thread.start()

    def cancel(self):
        self.flush()
        self._thread.stop()

    def flush(self):
        with self.mutex:
            if self._logs_buf:
                if self.remote:
                    good = append_logs_buf_to_remote(
                        self._logs_buf, self.name, self.metadata, self.post_addr)
                    if not good:
                        # failed to send logs, retry next time.
                        return
                if self.local_path:
                    append_logs_buf_to_local(self._logs_buf, self.local_path)
                self._logs_buf = list()

    def log(self, val):
        for f in self.memoize_funcs:
            if callable(f):
                f(self.memoize, val)
        self.append(val)

    def add_memoize_funcs(self, memoize_funcs):
        for f in memoize_funcs:
            self.memoize_funcs.append(f)

    def show_buffer(self):
        print(self._logs_buf)

    def show_user_buffer(self):
        print(self._user_buf)


if __name__ == "__main__":
    lg = Logger("fake", sample_time_interval_seconds=1,
                metadata={}, local_path="./what.json")
    lg.start()
    lg.log({"v1": 1, "v2": 2})
    for i in range(30):
        time.sleep(0.3)
        lg.log({"v1": i, "v2": 100+2*i})
        if i % 10 == 0:
            lg.log([{"fucker": 1*i}, {"fucker2": 2*i+100}])
        if i % 5 == 0:
            lg.show_buffer()

    lg.cancel()



# From WandB Client stat.py
import collections
import pynvml
import time
import os
from numbers import Number
import threading
import psutil


def gpu_in_use_by_this_process(gpu_handle):
    if not psutil:
        return False

    # NOTE: this optimizes for the case where wandb was initialized from
    # iniside the user script (i.e. `wandb.init()`). If we ran using
    # `wandb run` on the command line, the shell will be detected as the
    # parent, possible resulting in sibling processes being incorrectly
    # indentified as part of this process -- still better than not
    # detecting in-use gpus at all.
    base_process = psutil.Process().parent() or psutil.Process()

    our_processes = base_process.children(recursive=True)
    our_processes.append(base_process)

    our_pids = set([
        process.pid
        for process
        in our_processes
    ])

    compute_pids = set([
        process.pid
        for process
        in pynvml.nvmlDeviceGetComputeRunningProcesses(gpu_handle)
    ])
    graphics_pids = set([
        process.pid
        for process
        in pynvml.nvmlDeviceGetGraphicsRunningProcesses(gpu_handle)
    ])

    pids_using_device = compute_pids | graphics_pids

    return len(pids_using_device & our_pids) > 0


class SystemStats(object):
    def __init__(self, run):
        try:
            pynvml.nvmlInit()
            self.gpu_count = pynvml.nvmlDeviceGetCount()
        except pynvml.NVMLError as err:
            self.gpu_count = 0
        self.run = run
        self.sampler = {}
        self.samples = 0
        self._shutdown = False
        if not psutil:
            print(
                "psutil not installed, only GPU stats will be reported.  Install with pip install psutil")
        self._thread = threading.Thread(target=self._thread_body)
        self._thread.daemon = True

    def start(self):
        self._thread.start()

    @property
    def proc(self):
        return psutil.Process(pid=self.run.pid)

    @property
    def sample_rate_seconds(self):
        """Sample system stats every this many seconds, min is 3"""
        return max(2, self.run._sys_stat_sample_interval_seconds)

    @property
    def samples_to_average(self):
        """The number of samples to average before pushing, defaults to 15 valid range (2:30)"""
        return min(30, max(1, int(self.run._sys_stat_sample_size)))

    def _thread_body(self):
        while True:
            stats = self.stats()
            for stat, value in stats.items():
                if isinstance(value, Number):
                    self.sampler[stat] = self.sampler.get(stat, [])
                    self.sampler[stat].append(value)
            self.samples += 1
            if self._shutdown or self.samples >= self.samples_to_average:
                self.flush()
                if self._shutdown:
                    break
            seconds = 0
            while seconds < self.sample_rate_seconds:
                time.sleep(0.1)
                seconds += 0.1
                if self._shutdown:
                    break

    def shutdown(self):
        self._shutdown = True
        try:
            self._thread.join()
        # Incase we never start it
        except RuntimeError:
            pass

    def flush(self):
        stats = self.stats()
        for stat, value in stats.items():
            # TODO: a bit hacky, we assume all numbers should be averaged.  If you want
            # max for a stat, you must put it in a sub key, like ["network"]["sent"]
            if isinstance(value, Number):
                samples = list(self.sampler.get(stat, [stats[stat]]))
                stats[stat] = round(sum(samples) / len(samples), 2)
        # TODO: select on events?
        #       bind with query?
        self.run._loggers["system"].log(stats)
        self.samples = 0
        self.sampler = {}

    def stats(self):
        stats = {}
        for i in range(0, self.gpu_count):
            handle = pynvml.nvmlDeviceGetHandleByIndex(i)
            try:
                util = pynvml.nvmlDeviceGetUtilizationRates(handle)
                memory = pynvml.nvmlDeviceGetMemoryInfo(handle)
                temp = pynvml.nvmlDeviceGetTemperature(
                    handle, pynvml.NVML_TEMPERATURE_GPU)
                in_use_by_us = gpu_in_use_by_this_process(handle)

                stats["gpu.{}.{}".format(i, "gpu")] = util.gpu
                stats["gpu.{}.{}".format(i, "memory")] = util.memory
                stats["gpu.{}.{}".format(
                    i, "memoryAllocated")] = (memory.used / float(memory.total)) * 100
                stats["gpu.{}.{}".format(i, "temp")] = temp

                if in_use_by_us:
                    stats["gpu.process.{}.{}".format(i, "gpu")] = util.gpu
                    stats["gpu.process.{}.{}".format(
                        i, "memory")] = util.memory
                    stats["gpu.process.{}.{}".format(
                        i, "memoryAllocated")] = (memory.used / float(memory.total)) * 100
                    stats["gpu.process.{}.{}".format(i, "temp")] = temp

                    # Some GPUs don't provide information about power usage
                try:
                    power_watts = pynvml.nvmlDeviceGetPowerUsage(
                        handle) / 1000.0
                    power_capacity_watts = pynvml.nvmlDeviceGetEnforcedPowerLimit(
                        handle) / 1000.0
                    power_usage = (power_watts / power_capacity_watts) * 100

                    stats["gpu.{}.{}".format(i, "powerWatts")] = power_watts
                    stats["gpu.{}.{}".format(i, "powerPercent")] = power_usage

                    if in_use_by_us:
                        stats["gpu.process.{}.{}".format(
                            i, "powerWatts")] = power_watts
                        stats["gpu.process.{}.{}".format(
                            i, "powerPercent")] = power_usage

                except pynvml.NVMLError as err:
                    pass

            except pynvml.NVMLError as err:
                pass
        if psutil:
            #net = psutil.net_io_counters()
            sysmem = psutil.virtual_memory()
            stats["cpu"] = psutil.cpu_percent()
            stats["memory"] = sysmem.percent
        return stats



import setuptools

setuptools.setup(
    name="modelwhaleutils",
    version="0.4.9.0",
    author="modalwhale team",
    description="use in mw",
    url="https://github.com/Kesci/modelwhaleutils",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["PyJWT", "requests", "pynvml", "psutil"],
    python_requires='>=3.6',
)
